require "nats/client"

module Altaire

  module Gateway

    def self.start (urls, &block)
      NATS.start servers: [urls].flatten do
        @requests = {}
        NATS.subscribe "meta.cancel" do |msg|
          @requests[msg.to_i].raise(Interrupt, "Cancelled by gateway")
        rescue Exception
        end
        block.(self)
      end
    end

    def self.provide (subject, queue: "provider", interruptable: false, &block)
      NATS.subscribe subject, queue: queue do |msg, reply|
        msg = JSON.parse(msg)
        thread = Thread.new do
          NATS.publish reply, { result: block.(msg) }.to_json
        rescue => e
          NATS.publish reply, { error: e.message }.to_json
        ensure
          @requests.delete(msg["id"])
        end
        @requests[msg["id"]] = thread if interruptable
      end
    end

  end

end
